package com.dragonsoft.dcuc.approve.utils;

import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragoninfo.dcuc.common.enums.ErrorCodeEnum;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;

/**
 * 返回值工具类
 *
 * @author huangzqa
 * @date 2021/3/9
 **/
public class ResponseUtil {

    private ResponseUtil() {
    }

    /**
     * 判断返回值是否成功
     *
     * @param responseStatus 返回状态
     * @return 是否成功
     */
    public static boolean isSuccess(ResponseStatus responseStatus) {
        return responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE);
    }

    /**
     * 判断返回值是否失败
     *
     * @param responseStatus 返回状态
     * @return 是否失败
     */
    public static boolean isFail(ResponseStatus responseStatus) {
        return !isSuccess(responseStatus);
    }

    /**
     * 返回 ResponseDTO 错误信息
     *
     * @param message 消息
     * @return ResponseDTO
     */
    @SuppressWarnings("unchecked")
    public static <T> ResponseDTO<T> dtoFail(String message) {
        return ResponseDTO.fail(ResponseStatus.FAIL_CODE, message, null);
    }

    /**
     * 返回 ResponseDTO 信息
     *
     * @param statusCode 状态
     * @param message    消息
     * @return ResponseDTO
     */
    @SuppressWarnings("unchecked")
    public static <T> ResponseDTO<T> message(String statusCode, String message) {
        return ResponseDTO.fail(statusCode, message, null);
    }

    /**
     * 获取泛型返回值
     *
     * @param responseDTO 返回类
     * @param <T>         泛型
     * @return 返回值
     */
    @SuppressWarnings("unchecked")
    public static <T> T getResult(ResponseDTO<T> responseDTO) {
        return (T) responseDTO.getResult();
    }

    /**
     * 新实体
     *
     * @param result 结果
     * @param <T>    泛型
     * @return 结果
     */
    @SuppressWarnings("unchecked")
    public static <T> ResponseDTO<T> newInstance(T result) {
        return ResponseDTO.newInstance(result);
    }

    /**
     * 错误代码
     *
     * @param errorCodeEnum 错误枚举
     * @return 错误代码返回
     */
    public static ResponseStatus errorCode(ErrorCodeEnum errorCodeEnum) {
        return ResponseStatus.fail(errorCodeEnum.getValue(), errorCodeEnum.getLabel());
    }

    /**
     * 错误代码转换为JSON
     *
     * @param errorCodeEnum 错误枚举
     * @return 错误代码JSON
     */
    public static String errorCodeJson(ErrorCodeEnum errorCodeEnum) {
        return JsonUtils.toJSONString(errorCode(errorCodeEnum));
    }

    /**
     * 判断是否成功
     *
     * @param result 结果
     * @param <T>    泛型
     * @return 是否成功
     */
    public static <T> boolean isSuccess(Result<T> result) {
        String resultStatus = result.getResult();
        return resultStatus.equalsIgnoreCase(ResultEnum.SUCCESS.getValue());
    }

    /**
     * 判断返回值是否失败
     *
     * @param result 返回状态
     * @return 是否失败
     */
    public static <T> boolean isFail(Result<T> result) {
        return !isSuccess(result);
    }

    /**
     * 获取泛型返回值
     *
     * @param result 返回类
     * @param <T>    泛型
     * @return 返回值
     */
    @SuppressWarnings("unchecked")
    public static <T> T getContent(Result<T> result) {
        return (T) result.getContent();
    }

    /**
     * 判断是否成功
     *
     * @param result 结果
     * @param <T>    泛型
     * @return 是否成功
     */
    public static <T> boolean isSuccess(ApiResult result) {
        String statusCode = result.getStatusCode();
        return statusCode.equalsIgnoreCase(ResultEnum.SUCCESS.getValue());
    }

    /**
     * 判断返回值是否失败
     *
     * @param result 返回状态
     * @return 是否失败
     */
    public static <T> boolean isFail(ApiResult result) {
        return !isSuccess(result);
    }
}
